import typing, abc
from System import Array_1, Exception
from System.Collections.Generic import List_1
from TransformsAI.Animo import Actor, VoxelGrid, GridObject, HoldableObject, GridTransform, Directions, MoveResults
from System.Numerics import Vector2
from TransformsAI.Animo.Numerics import Vec3Int, Vec2Int
from TransformsAI.Animo.Tools import ReadOnlyList_1
from System.Collections import IDictionary
from System.Reflection import MethodBase
from TransformsAI.Animo.Constants import TypeIds

class ArcLaunchIntent(Intent):
    def __init__(self) -> None: ...
    DefaultBounceArc : Array_1[int]
    @property
    def BounceArc(self) -> Array_1[int]: ...
    @BounceArc.setter
    def BounceArc(self, value: Array_1[int]) -> Array_1[int]: ...
    @property
    def Bounces(self) -> List_1[ArcLaunchIntent.Bounce]: ...
    @property
    def CanBeCaughtOnInstantRicochet(self) -> bool: ...
    @CanBeCaughtOnInstantRicochet.setter
    def CanBeCaughtOnInstantRicochet(self, value: bool) -> bool: ...
    @property
    def Catcher(self) -> Actor: ...
    @Catcher.setter
    def Catcher(self, value: Actor) -> Actor: ...
    @property
    def ChildIntents(self) -> List_1[Intent]: ...
    @property
    def CurrentDirection(self) -> Vector2: ...
    @property
    def Grid(self) -> VoxelGrid: ...
    @property
    def IntentState(self) -> Intent.States: ...
    @property
    def MaxBounces(self) -> int: ...
    @MaxBounces.setter
    def MaxBounces(self, value: int) -> int: ...
    @property
    def MaxRicochets(self) -> int: ...
    @MaxRicochets.setter
    def MaxRicochets(self, value: int) -> int: ...
    @property
    def OriginalDirection(self) -> Vector2: ...
    @OriginalDirection.setter
    def OriginalDirection(self, value: Vector2) -> Vector2: ...
    @property
    def Path(self) -> List_1[Vec3Int]: ...
    @property
    def Priority(self) -> float: ...
    @property
    def Result(self) -> typing.Optional[ArcLaunchIntent.Results]: ...
    @Result.setter
    def Result(self, value: typing.Optional[ArcLaunchIntent.Results]) -> typing.Optional[ArcLaunchIntent.Results]: ...
    @property
    def Ricochets(self) -> List_1[ArcLaunchIntent.Ricochet]: ...
    @property
    def Source(self) -> GridObject: ...
    @property
    def ThrowArc(self) -> Array_1[int]: ...
    @ThrowArc.setter
    def ThrowArc(self, value: Array_1[int]) -> Array_1[int]: ...
    def CalculateArc(self, origin: Vec3Int, lifts: Array_1[int], direction: Vector2, outHeights: List_1[int], outCells: List_1[Vec2Int]) -> None: ...
    def GetMaxHeightAtCell(self, cell: Vec2Int) -> int: ...
    def Traverse(self, cell: Vec2Int, flyingObjectHeight: int, heightAtCell: int) -> ArcLaunchIntent.TraversalResults: ...

    class Bounce:
        @property
        def BouncedObjects(self) -> ReadOnlyList_1[GridObject]: ...
        @BouncedObjects.setter
        def BouncedObjects(self, value: ReadOnlyList_1[GridObject]) -> ReadOnlyList_1[GridObject]: ...
        @property
        def PathIndex(self) -> int: ...
        @PathIndex.setter
        def PathIndex(self, value: int) -> int: ...
        @property
        def Position(self) -> Vec3Int: ...
        @Position.setter
        def Position(self, value: Vec3Int) -> Vec3Int: ...


    class Results(typing.SupportsInt):
        @typing.overload
        def __init__(self, value : int) -> None: ...
        @typing.overload
        def __init__(self, value : int, force_if_true: bool) -> None: ...
        def __int__(self) -> int: ...
        
        # Values:
        Land : ArcLaunchIntent.Results # 0
        Catch : ArcLaunchIntent.Results # 1
        Pop : ArcLaunchIntent.Results # 2


    class Ricochet:
        @property
        def HitObject(self) -> GridObject: ...
        @HitObject.setter
        def HitObject(self, value: GridObject) -> GridObject: ...
        @property
        def PathIndex(self) -> int: ...
        @PathIndex.setter
        def PathIndex(self, value: int) -> int: ...
        @property
        def Position(self) -> Vec3Int: ...
        @Position.setter
        def Position(self, value: Vec3Int) -> Vec3Int: ...


    class TraversalResults(typing.SupportsInt):
        @typing.overload
        def __init__(self, value : int) -> None: ...
        @typing.overload
        def __init__(self, value : int, force_if_true: bool) -> None: ...
        def __int__(self) -> int: ...
        
        # Values:
        Through : ArcLaunchIntent.TraversalResults # 0
        Ricochet : ArcLaunchIntent.TraversalResults # 1
        BounceForward : ArcLaunchIntent.TraversalResults # 2
        Land : ArcLaunchIntent.TraversalResults # 3



class CategoryGroups(abc.ABC):
    Character : float
    Intrinsic : float
    Item : float


class FeedIntent(HeldUseIntent):
    def __init__(self) -> None: ...
    @property
    def ChildIntents(self) -> List_1[Intent]: ...
    @property
    def DidSucceed(self) -> bool: ...
    @property
    def FeedReceiver(self) -> GridObject: ...
    @FeedReceiver.setter
    def FeedReceiver(self, value: GridObject) -> GridObject: ...
    @property
    def Grid(self) -> VoxelGrid: ...
    @property
    def IntentState(self) -> Intent.States: ...
    @property
    def Priority(self) -> float: ...
    @property
    def Source(self) -> HoldableObject: ...
    @property
    def Source(self) -> GridObject: ...
    @property
    def User(self) -> Actor: ...
    @User.setter
    def User(self, value: Actor) -> Actor: ...


class ForceDropIntent(Intent):
    def __init__(self) -> None: ...
    @property
    def ChildIntents(self) -> List_1[Intent]: ...
    @property
    def DroppedObject(self) -> HoldableObject: ...
    @DroppedObject.setter
    def DroppedObject(self, value: HoldableObject) -> HoldableObject: ...
    @property
    def Grid(self) -> VoxelGrid: ...
    @property
    def IntentState(self) -> Intent.States: ...
    @property
    def Priority(self) -> float: ...
    @property
    def Result(self) -> ForceDropIntent.Results: ...
    @Result.setter
    def Result(self, value: ForceDropIntent.Results) -> ForceDropIntent.Results: ...
    @property
    def Source(self) -> Actor: ...
    @property
    def Source(self) -> GridObject: ...

    class Results(typing.SupportsInt):
        @typing.overload
        def __init__(self, value : int) -> None: ...
        @typing.overload
        def __init__(self, value : int, force_if_true: bool) -> None: ...
        def __int__(self) -> int: ...
        
        # Values:
        Drop : ForceDropIntent.Results # 0
        DestroyHeldObject : ForceDropIntent.Results # 1



class GrabIntent(Intent):
    def __init__(self) -> None: ...
    @property
    def ChildIntents(self) -> List_1[Intent]: ...
    @property
    def DroppedObject(self) -> HoldableObject: ...
    @DroppedObject.setter
    def DroppedObject(self, value: HoldableObject) -> HoldableObject: ...
    @property
    def GrabbedObject(self) -> HoldableObject: ...
    @GrabbedObject.setter
    def GrabbedObject(self, value: HoldableObject) -> HoldableObject: ...
    @property
    def Grid(self) -> VoxelGrid: ...
    @property
    def IntentState(self) -> Intent.States: ...
    @property
    def Priority(self) -> float: ...
    @property
    def Result(self) -> GrabIntent.Results: ...
    @Result.setter
    def Result(self, value: GrabIntent.Results) -> GrabIntent.Results: ...
    @property
    def Source(self) -> Actor: ...
    @property
    def Source(self) -> GridObject: ...

    class Results(typing.SupportsInt):
        @typing.overload
        def __init__(self, value : int) -> None: ...
        @typing.overload
        def __init__(self, value : int, force_if_true: bool) -> None: ...
        def __int__(self) -> int: ...
        
        # Values:
        GrabFail : GrabIntent.Results # 0
        Grab : GrabIntent.Results # 1
        Drop : GrabIntent.Results # 2
        Swap : GrabIntent.Results # 3
        DropFail : GrabIntent.Results # 4



class GrabPriorities(abc.ABC):
    Default : float


class HeldUseIntent(UseIntent):
    @property
    def ChildIntents(self) -> List_1[Intent]: ...
    @property
    def DidSucceed(self) -> bool: ...
    @property
    def Grid(self) -> VoxelGrid: ...
    @property
    def IntentState(self) -> Intent.States: ...
    @property
    def Priority(self) -> float: ...
    @property
    def Source(self) -> HoldableObject: ...
    @property
    def Source(self) -> GridObject: ...
    @property
    def User(self) -> Actor: ...
    @User.setter
    def User(self, value: Actor) -> Actor: ...


class Intent(abc.ABC):
    @property
    def ChildIntents(self) -> List_1[Intent]: ...
    @property
    def Grid(self) -> VoxelGrid: ...
    @Grid.setter
    def Grid(self, value: VoxelGrid) -> VoxelGrid: ...
    @property
    def IntentState(self) -> Intent.States: ...
    @IntentState.setter
    def IntentState(self, value: Intent.States) -> Intent.States: ...
    @property
    def Priority(self) -> float: ...
    @property
    def Source(self) -> GridObject: ...
    @Source.setter
    def Source(self, value: GridObject) -> GridObject: ...
    def CalculateAffectedCells(self, outAffectedCells: List_1[Vec2Int]) -> None: ...
    def Cancel(self) -> None: ...
    def Execute(self) -> None: ...
    def Recycle(self) -> None: ...
    def WasCausedBy(self, causer: GridObject) -> bool: ...
    # Skipped LeaseFromPool due to it being static, abstract and generic.

    LeaseFromPool : LeaseFromPool_MethodGroup
    class LeaseFromPool_MethodGroup:
        def __getitem__(self, t:typing.Type[LeaseFromPool_1_T1]) -> LeaseFromPool_1[LeaseFromPool_1_T1]: ...

        LeaseFromPool_1_T1 = typing.TypeVar('LeaseFromPool_1_T1')
        class LeaseFromPool_1(typing.Generic[LeaseFromPool_1_T1]):
            LeaseFromPool_1_T = Intent.LeaseFromPool_MethodGroup.LeaseFromPool_1_T1
            def __call__(self, grid: VoxelGrid, source: GridObject) -> LeaseFromPool_1_T:...



    class States(typing.SupportsInt):
        @typing.overload
        def __init__(self, value : int) -> None: ...
        @typing.overload
        def __init__(self, value : int, force_if_true: bool) -> None: ...
        def __int__(self) -> int: ...
        
        # Values:
        Reset : Intent.States # 0
        Executed : Intent.States # 1
        Cancelled : Intent.States # 2



class IntentPriorities(abc.ABC):
    ArcLaunch : float
    Burn : float
    CharacterWalk : float
    CollectCrystal : float
    DestroyBlock : float
    ExtinguishTorch : float
    ForceDrop : float
    GiftFlowerSeed : float
    GrabObject : float
    Hug : float
    LightTorch : float
    MakeBlock : float
    MakeFlowerSeed : float
    OnCharacterStateChanged : float
    OnDestroyed : float
    OnLifted : float
    OnMoved : float
    OnPlaced : float
    Paint : float
    Plant : float
    PushLargeSnowball : float
    PushObject : float
    ShakeTreeWithFruits : float
    Slash : float
    Smack : float
    SpawnTreeFruits : float
    ThrowSmallSnowball : float
    TransformObject : float
    UseFailed : float
    Watering : float
    @classmethod
    @property
    def BallRoll(cls) -> float: ...
    @classmethod
    @BallRoll.setter
    def BallRoll(cls, value: float) -> float: ...
    @classmethod
    @property
    def FeedTreeFruit(cls) -> float: ...
    @classmethod
    @FeedTreeFruit.setter
    def FeedTreeFruit(cls, value: float) -> float: ...
    @classmethod
    @property
    def KickBall(cls) -> float: ...
    @classmethod
    @KickBall.setter
    def KickBall(cls, value: float) -> float: ...
    @classmethod
    @property
    def OnCreatedIntent(cls) -> float: ...
    @classmethod
    @OnCreatedIntent.setter
    def OnCreatedIntent(cls, value: float) -> float: ...
    @classmethod
    @property
    def TakeSnowball(cls) -> float: ...
    @classmethod
    @TakeSnowball.setter
    def TakeSnowball(cls, value: float) -> float: ...
    @classmethod
    @property
    def Talk(cls) -> float: ...
    @classmethod
    @Talk.setter
    def Talk(cls, value: float) -> float: ...
    @classmethod
    @property
    def ThrowBall(cls) -> float: ...
    @classmethod
    @ThrowBall.setter
    def ThrowBall(cls, value: float) -> float: ...
    @staticmethod
    def EarlyToLateComparer(x: Intent, y: Intent) -> int: ...


class InvalidIntentException(Exception):
    def __init__(self, intent: Intent, propertyName: str, message: str = ...) -> None: ...
    @property
    def Data(self) -> IDictionary: ...
    @property
    def HelpLink(self) -> str: ...
    @HelpLink.setter
    def HelpLink(self, value: str) -> str: ...
    @property
    def HResult(self) -> int: ...
    @HResult.setter
    def HResult(self, value: int) -> int: ...
    @property
    def InnerException(self) -> Exception: ...
    @property
    def Message(self) -> str: ...
    @property
    def Source(self) -> str: ...
    @Source.setter
    def Source(self, value: str) -> str: ...
    @property
    def StackTrace(self) -> str: ...
    @property
    def TargetSite(self) -> MethodBase: ...


class OnCreatedIntent(Intent):
    def __init__(self) -> None: ...
    @property
    def ChildIntents(self) -> List_1[Intent]: ...
    @property
    def Grid(self) -> VoxelGrid: ...
    @property
    def IntentState(self) -> Intent.States: ...
    @property
    def Priority(self) -> float: ...
    @property
    def Source(self) -> GridObject: ...


class OnDestroyedIntent(Intent):
    def __init__(self) -> None: ...
    @property
    def ChildIntents(self) -> List_1[Intent]: ...
    @property
    def Grid(self) -> VoxelGrid: ...
    @property
    def IntentState(self) -> Intent.States: ...
    @property
    def Priority(self) -> float: ...
    @property
    def Source(self) -> GridObject: ...


class OnMovedIntent(Intent):
    def __init__(self) -> None: ...
    @property
    def ChildIntents(self) -> List_1[Intent]: ...
    @property
    def Grid(self) -> VoxelGrid: ...
    @property
    def IntentState(self) -> Intent.States: ...
    @property
    def OriginTransform(self) -> typing.Optional[GridTransform]: ...
    @OriginTransform.setter
    def OriginTransform(self, value: typing.Optional[GridTransform]) -> typing.Optional[GridTransform]: ...
    @property
    def Priority(self) -> float: ...
    @property
    def Source(self) -> GridObject: ...
    @property
    def WasAddedToGrid(self) -> bool: ...
    @property
    def WasRemovedFromGrid(self) -> bool: ...


class OnUseFailedIntent(Intent):
    def __init__(self) -> None: ...
    @property
    def ChildIntents(self) -> List_1[Intent]: ...
    @property
    def Grid(self) -> VoxelGrid: ...
    @property
    def IntentState(self) -> Intent.States: ...
    @property
    def Priority(self) -> float: ...
    @property
    def Source(self) -> GridObject: ...


class PlantIntent(HeldUseIntent):
    def __init__(self) -> None: ...
    @property
    def ChildIntents(self) -> List_1[Intent]: ...
    @property
    def DidSucceed(self) -> bool: ...
    @property
    def Grid(self) -> VoxelGrid: ...
    @property
    def IntentState(self) -> Intent.States: ...
    @property
    def NewPlant(self) -> GridObject: ...
    @NewPlant.setter
    def NewPlant(self, value: GridObject) -> GridObject: ...
    @property
    def PlantPosition(self) -> typing.Optional[Vec3Int]: ...
    @PlantPosition.setter
    def PlantPosition(self, value: typing.Optional[Vec3Int]) -> typing.Optional[Vec3Int]: ...
    @property
    def Priority(self) -> float: ...
    @property
    def Result(self) -> PlantIntent.Results: ...
    @property
    def Source(self) -> HoldableObject: ...
    @property
    def Source(self) -> GridObject: ...
    @property
    def TargetPlantType(self) -> typing.Optional[TypeIds]: ...
    @TargetPlantType.setter
    def TargetPlantType(self, value: typing.Optional[TypeIds]) -> typing.Optional[TypeIds]: ...
    @property
    def User(self) -> Actor: ...
    @User.setter
    def User(self, value: Actor) -> Actor: ...

    class Results(typing.SupportsInt):
        @typing.overload
        def __init__(self, value : int) -> None: ...
        @typing.overload
        def __init__(self, value : int, force_if_true: bool) -> None: ...
        def __int__(self) -> int: ...
        
        # Values:
        Fail : PlantIntent.Results # 0
        Plant : PlantIntent.Results # 1



class PlantSelfIntent(HeldUseIntent):
    def __init__(self) -> None: ...
    @property
    def ChildIntents(self) -> List_1[Intent]: ...
    @property
    def DidSucceed(self) -> bool: ...
    @property
    def Grid(self) -> VoxelGrid: ...
    @property
    def IntentState(self) -> Intent.States: ...
    @property
    def PlantPosition(self) -> typing.Optional[Vec3Int]: ...
    @PlantPosition.setter
    def PlantPosition(self, value: typing.Optional[Vec3Int]) -> typing.Optional[Vec3Int]: ...
    @property
    def Priority(self) -> float: ...
    @property
    def Source(self) -> HoldableObject: ...
    @property
    def Source(self) -> GridObject: ...
    @property
    def User(self) -> Actor: ...
    @User.setter
    def User(self, value: Actor) -> Actor: ...


class PushIntent(Intent):
    def __init__(self) -> None: ...
    @property
    def ChildIntents(self) -> List_1[Intent]: ...
    @property
    def Grid(self) -> VoxelGrid: ...
    @property
    def IntentState(self) -> Intent.States: ...
    @property
    def Priority(self) -> float: ...
    @property
    def PushDirection(self) -> Vec2Int: ...
    @PushDirection.setter
    def PushDirection(self, value: Vec2Int) -> Vec2Int: ...
    @property
    def Result(self) -> PushIntent.Results: ...
    @Result.setter
    def Result(self, value: PushIntent.Results) -> PushIntent.Results: ...
    @property
    def Source(self) -> GridObject: ...

    class Results(typing.SupportsInt):
        @typing.overload
        def __init__(self, value : int) -> None: ...
        @typing.overload
        def __init__(self, value : int, force_if_true: bool) -> None: ...
        def __int__(self) -> int: ...
        
        # Values:
        Fail : PushIntent.Results # 0
        Push : PushIntent.Results # 1



class TransformObjectIntent(Intent):
    def __init__(self) -> None: ...
    @property
    def ChildIntents(self) -> List_1[Intent]: ...
    @property
    def Grid(self) -> VoxelGrid: ...
    @property
    def IntentState(self) -> Intent.States: ...
    @property
    def NewObject(self) -> GridObject: ...
    @NewObject.setter
    def NewObject(self, value: GridObject) -> GridObject: ...
    @property
    def Priority(self) -> float: ...
    @property
    def Source(self) -> GridObject: ...
    @property
    def TargetType(self) -> typing.Optional[TypeIds]: ...
    @TargetType.setter
    def TargetType(self, value: typing.Optional[TypeIds]) -> typing.Optional[TypeIds]: ...
    @property
    def VariantId(self) -> typing.Optional[int]: ...
    @VariantId.setter
    def VariantId(self, value: typing.Optional[int]) -> typing.Optional[int]: ...


class TurnIntent(Intent):
    def __init__(self) -> None: ...
    @property
    def ChildIntents(self) -> List_1[Intent]: ...
    @property
    def Direction(self) -> Directions: ...
    @Direction.setter
    def Direction(self, value: Directions) -> Directions: ...
    @property
    def Grid(self) -> VoxelGrid: ...
    @property
    def IntentState(self) -> Intent.States: ...
    @property
    def Priority(self) -> float: ...
    @property
    def Result(self) -> TurnIntent.Results: ...
    @Result.setter
    def Result(self, value: TurnIntent.Results) -> TurnIntent.Results: ...
    @property
    def Source(self) -> Actor: ...
    @property
    def Source(self) -> GridObject: ...

    class Results(typing.SupportsInt):
        @typing.overload
        def __init__(self, value : int) -> None: ...
        @typing.overload
        def __init__(self, value : int, force_if_true: bool) -> None: ...
        def __int__(self) -> int: ...
        
        # Values:
        Fail : TurnIntent.Results # 0
        Turned : TurnIntent.Results # 1



class UseIntent(Intent):
    @property
    def ChildIntents(self) -> List_1[Intent]: ...
    @property
    def DidSucceed(self) -> bool: ...
    @property
    def Grid(self) -> VoxelGrid: ...
    @property
    def IntentState(self) -> Intent.States: ...
    @property
    def Priority(self) -> float: ...
    @property
    def Source(self) -> GridObject: ...
    @property
    def User(self) -> Actor: ...
    @User.setter
    def User(self, value: Actor) -> Actor: ...


class WalkIntent(Intent):
    def __init__(self) -> None: ...
    @property
    def ChildIntents(self) -> List_1[Intent]: ...
    @property
    def Grid(self) -> VoxelGrid: ...
    @property
    def IntentState(self) -> Intent.States: ...
    @property
    def MoveResult(self) -> MoveResults: ...
    @MoveResult.setter
    def MoveResult(self, value: MoveResults) -> MoveResults: ...
    @property
    def Priority(self) -> float: ...
    @property
    def Source(self) -> Actor: ...
    @property
    def Source(self) -> GridObject: ...

