import typing
from TransformsAI.Animo.Intents import Intent, UseIntent
from System.Collections.Generic import List_1
from TransformsAI.Animo import VoxelGrid, GridObject, ObjectInfo, ObjectGroup, GridTransform
from System import IEquatable_1, Array_1, Guid, ValueTuple_2
from TransformsAI.Animo.Constants import TypeIds, Effects
from TransformsAI.Animo.Numerics import Vec2Int, Vec3Int
from TransformsAI.Animo.Rewards.Categories import RewardCategory
from TransformsAI.Animo.Rewards import ObjectDestroyedReward, Reward
from TransformsAI.Animo.Tools import ReadOnlyArray_1

class BurnIntent(Intent):
    def __init__(self) -> None: ...
    @property
    def ChildIntents(self) -> List_1[Intent]: ...
    @property
    def Grid(self) -> VoxelGrid: ...
    @property
    def IntentState(self) -> Intent.States: ...
    @property
    def Priority(self) -> float: ...
    @property
    def Source(self) -> FireObject: ...
    @property
    def Source(self) -> GridObject: ...


class FireInfo(ObjectInfo, IEquatable_1[FireInfo]):
    def __init__(self) -> None: ...
    @property
    def CanStepOnObjects(self) -> bool: ...
    @property
    def CreatesObjects(self) -> Array_1[TypeIds]: ...
    @property
    def IsDestroyedInDeepWater(self) -> bool: ...
    @property
    def IsDestroyedInShallowWater(self) -> bool: ...
    @property
    def IsDestructible(self) -> bool: ...
    @property
    def IsGrabbable(self) -> bool: ...
    @property
    def IsOccupier(self) -> bool: ...
    @property
    def IsUsableOnGrid(self) -> bool: ...
    @property
    def MaximumBurnAmount(self) -> int: ...
    @property
    def ObjectGroup(self) -> ObjectGroup: ...
    @property
    def TransformsInto(self) -> Array_1[TypeIds]: ...
    @property
    def Type(self) -> typing.Type[typing.Any]: ...
    @property
    def TypeId(self) -> TypeIds: ...
    @property
    def UsePriority(self) -> float: ...
    def CanReceiveEffect(self, effect: Effects) -> bool: ...
    def CreateObject(self) -> GridObject: ...
    def GetHashCode(self) -> int: ...
    def __eq__(self, left: FireInfo, right: FireInfo) -> bool: ...
    def __ne__(self, left: FireInfo, right: FireInfo) -> bool: ...
    def ToString(self) -> str: ...
    # Skipped Equals due to it being static, abstract and generic.

    Equals : Equals_MethodGroup
    class Equals_MethodGroup:
        @typing.overload
        def __call__(self, other: FireInfo) -> bool:...
        @typing.overload
        def __call__(self, other: ObjectInfo) -> bool:...
        @typing.overload
        def __call__(self, obj: typing.Any) -> bool:...



class FireObject(GridObject):
    def __init__(self) -> None: ...
    Info : ObjectInfo
    Intensity : int
    MaximumIntensity : int
    SpreadInterval : int
    SpreadTimer : int
    @property
    def BurnAmount(self) -> int: ...
    @property
    def Cell(self) -> Vec2Int: ...
    @property
    def Container(self) -> GridObject: ...
    @Container.setter
    def Container(self, value: GridObject) -> GridObject: ...
    @property
    def Direction(self) -> Vec2Int: ...
    @property
    def Grid(self) -> VoxelGrid: ...
    @property
    def InstanceId(self) -> Guid: ...
    @property
    def IntentsFiredThisTurn(self) -> List_1[Intent]: ...
    @property
    def IsDestroyed(self) -> bool: ...
    @property
    def IsOnGrid(self) -> bool: ...
    @property
    def IsStoodUpon(self) -> bool: ...
    @property
    def MainIntent(self) -> Intent: ...
    @property
    def Position(self) -> Vec3Int: ...
    @property
    def PushedPosition(self) -> typing.Optional[Vec3Int]: ...
    @PushedPosition.setter
    def PushedPosition(self, value: typing.Optional[Vec3Int]) -> typing.Optional[Vec3Int]: ...
    @property
    def StateObservation(self) -> ValueTuple_2[float, float]: ...
    @property
    def Transform(self) -> GridTransform: ...
    @property
    def TransformOnGrid(self) -> None: ...
    @TransformOnGrid.setter
    def TransformOnGrid(self, value: typing.Optional[GridTransform]) -> None: ...
    @property
    def TypeId(self) -> TypeIds: ...
    @property
    def UsedIntent(self) -> UseIntent: ...
    @property
    def VariantId(self) -> int: ...
    @VariantId.setter
    def VariantId(self, value: int) -> int: ...
    def OnEffect(self, effect: Effects, source: GridObject, outIntents: List_1[Intent]) -> bool: ...
    def OnOverlapStarted(self, outIntents: List_1[Intent], overlappingObject: GridObject) -> None: ...


class FireRewards(RewardCategory):
    def __init__(self) -> None: ...
    Extinguished : ObjectDestroyedReward
    name : str
    @property
    def AssociatedObject(self) -> typing.Optional[TypeIds]: ...
    @property
    def CategoryId(self) -> str: ...
    @property
    def Rewards(self) -> ReadOnlyArray_1[Reward]: ...

